@@ -1,5 +1,9 @@
 from uncertainties.umath import acos, radians, degrees, sin, cos
+import astropy.constants as c
+import astropy.units as u
+import numpy as np
 
+rsun = c.R_sun.to( u.km).value
 
 def P_to_aR(P,R,M,format='days'):
     """
@@ -22,8 +26,8 @@ def P_to_aR(P,R,M,format='days'):
     aR: Scaled semi-major axis.
     
     """
-    AU_factor=1.496e8/(R*695510)
-    if format=='days':
+    AU_factor = c.au.to(u.km).value/(R*rsun)
+    if format == 'days':
         P=P/365.
         
     return P**(2/3.)*M**(1/3.)*AU_factor
@@ -45,11 +49,11 @@ def AU_to_aR(AU,R):
     aR: Scaled semi-major axis.
     
     """
-    return AU*1.496e8/(R*695510)
+    return AU*c.au.to(u.km).value / (R*rsun)
 
     
 
-def impact_parameter(inc,a,format='deg',e=0,w=90):
+def impact_parameter(inc, a, e=0, w=90, format='deg'):
     """
     Function to convert inclination to impact parameter b.
     input format of angles as 'deg' or 'rad'.
@@ -57,24 +61,36 @@ def impact_parameter(inc,a,format='deg',e=0,w=90):
     
     Parameters:
     ----------
-    inc: inclination of the planetary orbit
+    inc: float;
+        inclination of the planetary orbit
     
-    a: scaled semi-major axis in units of solar radii    
+    a: float;
+        scaled semi-major axis in units of solar radii 
     
-    format: (str) unit of inclincation angle  - "deg" or "rad"
+    e: float;
+        eccentricity of the orbit.
+    
+    w: float;
+        longitude of periastron
+      
+    
+    format: str;  - "deg" or "rad"
+        unit of the `inc` and `w`  
     
     Returns
     -------
     
     b: impact parameter
     """
-    ecc_factor=(1-e**2)/(1+e*sin(radians(w)))  
+
     if format == 'deg':
         inc = radians(inc)
+        w = radians(w)
 
+    ecc_factor=(1-e**2)/(1+e*sin(w))  
     return a*cos(inc)*ecc_factor
 
-def inclination(b,a,e=0,w=90):
+def inclination(b, a, e=0, w=90):
     """
     Function to convert impact parameter b to inclination in degrees.
     
@@ -83,7 +99,13 @@ def inclination(b,a,e=0,w=90):
     ----------
     b: Impact parameter of the transit.
     
-    a: Scaled semi-major axis i.e. a/R*
+    a: Scaled semi-major axis i.e. a/R*.
+
+    e: float;
+        eccentricity of the orbit.
+    
+    w: float;
+        longitude of periastron in degrees
     
     Returns
     --------
@@ -95,7 +117,7 @@ def inclination(b,a,e=0,w=90):
     inc = degrees(acos( float(b) / (a*ecc_factor)) )
     return round(inc, 2)
     
-def vsini(prot,st_rad):
+def vsini(prot, st_rad):
     """
     Function to convert stellar rotation period to vsini in km/s.
     
@@ -114,11 +136,11 @@ def vsini(prot,st_rad):
     """
     import numpy as np
     prot=np.array(prot)
-    vsini=(2*np.pi*st_rad*695510.)/(prot*24*60*60)
+    vsini=(2*np.pi*st_rad*rsun)/(prot*24*60*60)
     
     return vsini
 
-def prot(vsini,st_rad):
+def prot(vsini, st_rad):
     """
     Function to convert stellar rotation velocity vsini in km/s to rotation period in days.
     
@@ -137,7 +159,7 @@ def prot(vsini,st_rad):
     import numpy as np
     vsini=np.array(vsini)
 
-    prot=(2*np.pi*st_rad*695510.)/(vsini*24*60*60)
+    prot=(2*np.pi*st_rad*rsun)/(vsini*24*60*60)
     
     return prot
     
@@ -314,11 +336,103 @@ def rho_to_aR(rho,P):
         The scaled semi-major axis of the planet.
     """
     
-    import astropy.constants as c
-    import astropy.units as u
-    import numpy as np
     G = (c.G.to(u.cm**3/(u.g*u.second**2))).value
     Ps = P*(u.day.to(u.second))
     aR = ((rho*G*Ps**2)/(3*np.pi)) **(1/3.)
     return aR
     
+    
+   
+
+def timeperi_to_timetrans(tp, per, ecc, omega, secondary=False):
+    """
+    Convert Time of Periastron passage `tp` to Time of Transit i.e time of conjunction `tc`.
+    For circular orbits with ecc=0 and w=90. tc = tp.
+    Adopted from radvel: https://github.com/California-Planet-Search/radvel
+    see also: http://www.sternwarte.uni-erlangen.de/~hanke/science/VelaX-1/ellipticalOrbits.pdf
+
+    
+    Parameters:
+    -----------
+    tp: float;
+        time of periastron passage
+        
+    per: float;
+        Planet period in same unit as tp. if tp is a phase then per = 1.
+        
+    ecc: float;
+        eccentricity
+        
+    omega (float): argument of periastron (degrees)
+        
+    secondary: bool; 
+        calculate time of secondary eclipse instead
+    
+    Returns:
+    -------
+    
+    tc: float; 
+        time of inferior conjunction (time of transit if system is transiting)
+    """
+    try:
+        if ecc >= 1:
+            return tp
+    except ValueError:
+        pass
+
+    if secondary:
+        f = 3*np.pi/2 - omega*np.pi/180.                                       # true anomaly during secondary eclipse
+        ee = 2 * np.arctan(np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)))  # eccentric anomaly
+
+        # ensure that ee is between 0 and 2*pi (always the eclipse AFTER tp)
+        if isinstance(ee, np.float64):
+            ee = ee + 2 * np.pi
+        else:
+            ee[ee < 0.0] = ee + 2 * np.pi
+    else:
+        #time of conjunction is time when true anomaly is pi/2. - omega
+        f = np.pi/2 - omega*np.pi/180                       
+        ee = 2 * np.arctan(np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)))  # eccentric anomaly
+
+    tc = tp + per/(2*np.pi) * (ee - ecc*np.sin(ee))         # time of conjunction
+
+    return tc
+
+
+def timetrans_to_timeperi(tc, per, ecc, omega):
+    """
+    Convert Time of Transit (conjunction/midtransit) to Time of Periastron Passage (Taken from radvel)
+    https://github.com/California-Planet-Search/radvel
+    
+    Parameters:
+    ----------
+    tc: float;
+        time of midtransit (conjunction)
+    
+    per: float;
+        period in same unit as tc. if tc is a phase, use per = 1
+    
+    ecc: float;
+        eccentricity
+    
+    omega: float;
+        longitude of periastron (degrees)
+    
+    Returns:
+    --------
+    float: time of periastron passage
+    
+    """
+    try:
+        if ecc >= 1:
+            return tc
+    except ValueError:
+        pass
+
+    f = np.pi/2 - omega*np.pi/180
+    ee = 2 * np.arctan(np.tan(f/2) * np.sqrt((1-ecc)/(1+ecc)))  # eccentric anomaly
+    tp = tc - per/(2*np.pi) * (ee - ecc*np.sin(ee))      # time of periastron
+
+    return tp
+
+    
